
// openpluginlib - A plugin interface to openlibraries.

// Copyright (C) 2005-2006 Visual Media FX Ltd.
// Released under the LGPL.
// For more information, see http://www.openlibraries.org.

#include "subject.hpp"

// framework
#include "observer.hpp"

// std
#include <map>
#include <algorithm>
#include <iostream>

namespace olib { namespace openpluginlib { namespace pcos {

class subject::subject_impl
{
public:
    subject_impl()
        {}

    /// a map of observer, lock count
    typedef std::map< boost::shared_ptr< observer >, unsigned int > observer_container;
    observer_container observers;
};

subject::subject()
    : m_impl( new subject_impl() )
{
    // std::cout << "subject::CTOR: " << this << "\n";
}

subject::~subject()
{
    // std::cout << "subject::DTOR: " << this << "\n";
}

void subject::attach( boost::shared_ptr< observer > obs )
{
    // std::cout << "subject::attach: " << obs.get() << "\n";
    m_impl->observers.insert( std::make_pair( obs, 0 ) );
}

void subject::detach( boost::shared_ptr< observer > obs )
{
    subject_impl::observer_container::iterator I = m_impl->observers.find( obs );
    if ( I == m_impl->observers.end() )
    {
        return;
    }

    m_impl->observers.erase( I );
}

template < typename T > struct Notify
{
    Notify( subject* s )
        : m_subject( s )
        {}

    typedef typename T::value_type value_type;

    void operator()( const value_type& v )
    {
        if ( v.second )
        {
            // we have a non-zero block count
            // std::cout << "blocked observer found " << v.first.get() << "\n";
            return;
        }

        // std::cout << "updating observer " << v.first.get() << "\n";
        v.first->updated( m_subject );
    }

    subject* m_subject;
};

void subject::update()
{
    std::for_each( m_impl->observers.begin(), m_impl->observers.end(), Notify< subject_impl::observer_container >( this ) );
}

void subject::block( boost::shared_ptr< observer > wp )
{
    if ( !m_impl->observers.count( wp ) )
    {
        return;
    }

    m_impl->observers[ wp ]++;
}

void subject::unblock( boost::shared_ptr< observer > wp )
{
    if ( !m_impl->observers.count( wp ) )
    {
        return;
    }

    m_impl->observers[ wp ]--;
}


} } }
